<?php



require_once 'Classes/Pulsar/ScrunchingParameters.php';
require_once 'Classes/Pulsar/CPSR2Combiner.php';



/**
 * @class ObservationData
 * 
 * This class represents the data from a single observation, on disk
 * this is one or more archives. We use this interface to create a standard
 * way of querying to see how large the file will be with or without
 * scrunching.
 **/
class ObservationData
{
  // PUBLIC

  public function __construct()
  {
    $this->archives = Array();
    $this->scrunching_parameters = null;
    $this->path = null;
  }
  
  public function SetScrunchingParameters( ScrunchingParameters $s_scrunching_parameters )
  {
    if( $s_scrunching_parameters == null )
      throw new GZ_Exception( 'attempt to set scrunching parameters with null', 'ObservationData::SetScrunchingParameters' );
    
    $this->scrunching_parameters = $s_scrunching_parameters;
  }
  public function GetScrunchingParameters() { return $this->scrunching_parameters; }
  
  public function SetPath( $s_path ) { $this->path = $s_path; $this->ExamineObservation(); }
  public function GetPath() { return $this->path; }
  
  public function SetNSub( $s_nsub ) { $this->nsub = $s_nsub; }
  public function setNPol( $s_npol ) { $this->npol = $s_npol; }
  public function SetNBin( $s_nbin ) { $this->nbin = $s_nbin; }
  public function SetNChan( $s_nchan ) { $this->nchan = $s_nchan; }
  
  //! Make an estimate of the size the downloadable file will be
  public function EstimateDownloadableSize()
  {
    /*
     * if we have a single file
     *   return the size of that file
     * otherwise
     *   estimate what the file size will be after psradding them together
     *
     * if we have scrunching parameters
     *   estimate what the file size will be after scrunching
     **/
    $file_size = 0;
    
    if( count( $this->archives ) == 1 )
    {
      $file_size = $this->archives[0]->GetFileSize();
    }
    else
    {
      foreach( $this->archives as $next_archive )
      {
        $next_size = $next_archive->GetFileSize();
        // TODO guess the data size from the archive parameters npol,nbin,nchan,nsub
        $file_size += $next_size;
      }
    }
    
    if( $this->scrunching_parameters == null )
      throw new GZ_Exception( 'scrunching paramters not set', 'ObservationData::EstimateDownloadableSize' );
    
    if( $this->scrunching_parameters->ScrunchingSet() )
    {
      // get the number of subints,bins,channels,polarizations
      // figure out the data size from that
      // get the header size as file size - data size
      // determine the new post scrunching data size
      // add the header size to the post scrunching data size
          
      if( $this->nsub == 0 )
          throw new GZ_Exception( 'nsub is empty','ObservationData::EstimateDownloadableSize' );
      
      if( $this->nbin == 0 )
        throw new GZ_Exception( 'nbin is zero', 'ObservationData::EstimateDownloadableSize' );
      
      if( $this->npol == 0 )
        throw new GZ_Exception( 'npol is zero', 'ObservationData::EstimateDownloadableSize' );
      
      if( $this->nchan == 0 )
        throw new GZ_Exception( 'nchan is zero', 'ObservationData::EstimateDownloadableSize' ); 
          
      // correct nsub, should match the number of files
      if( count( $this->archives ) > 0 )
        $this->nsub = count( $this->archives );
      
      echo 'actual number of subins is ' . $this->nsub . ' <br>';
      
      $current_data_size = 2 * $this->nsub * $this->nbin * $this->nchan * $this->npol;
      echo 'current data size is ' . $current_data_size . '<br> ';
      
      $new_nsub = $this->nsub;
      $scr_nsub = $this->scrunching_parameters->GetTScrunch();
      echo 'scrunching every ' . $scr_nsub . ' subints toegether <br>';
      if( $scr_nsub != null )
      {
        $new_nsub = ceil( $new_nsub / $scr_nsub );
      }
      echo 'number of subints after scrunching will be ' . $new_nsub , '<br>';
      
    }
    
    return $file_size;
  }

  //! This function will psradd and scrunch a file in order to create the actual file for downloading
  public function MakeDownloadableFile()
  {
   /*
    * if we have multiple files
    *   psradd the files into one in the data area
    * otherwise
    *   create a link to the file in the data area
    *
    * if we are scrunching
    *   create the scrunched file in the data dir
    *
    * return a file object pointing to the created file
    */

   if( $this->path->IsDir() )
   {
     $combiner = new CPSR2Combiner();
     $combiner->SetSrcDir( $this->path );
     $combiner->SetDestFile( $target_file );
     $combiner->Execute();
   }
   else
   {
     if( count( $this->archives ) == 0 )
       throw new GZ_Exception( 'dont have an archive entry for single file archive', 'ObservationData::MakeDownloadableFile' );

     $pre_scr_file->SetPath( $this->archives[0]->GetPath() );
     $pre_scr_file->SetFilename( $this->archives[0]->GetFilename() );
   }
  }
  

  
  // PRIVATE
  
  private $archives;
  private $scrunching_parameters;
  private $path;
  private $nsub;
  private $nbin;
  private $nchan;
  private $npol;
  
  private function ExamineObservation()
  {
    /*
     * if the path is pointing to a directory
     *   for each file in the directory
     *     create a new archive object for that file
     * otherwise
     *   create a new archive object and point it at that file
     */
    if( $this->path == null )
        throw new GZ_Exception( 'No path set', 'ObservationData::ExamineObservation' );
    
    if( $this->path->IsDir() )
    {
      $dir_path = $this->path->GetFullPath();
      $dir = new Dir( $dir_path );
      foreach( $dir->ListFiles() as $next_filename )
      {
	$extension = strword( $next_filename, 1, '.' );
	if( $extension == 'cfb' || $extension == 'fb' )
	{
	  $new_archive = new File( $dir_path, $next_filename);
	  $this->archives[] = $new_archive;
	}
      }
    }
    else
    {
      $new_archive = new File( $this->path->GetPath(), $this->path->GetFilename() );
      $this->archives[] = $new_archive;
    }
  }
};

?>